<?php
/**
 * The MIMP_Folder:: class provides a set of methods for dealing with
 * folders, accounting for subscription, errors, etc.
 *
 * $Horde: mimp/lib/Folder.php,v 1.14 2005/01/03 14:35:16 jan Exp $
 *
 * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 2000-2005 Jon Parise <jon@csh.rit.edu>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.14 $
 * @since   MIMP 0.0.1
 * @package MIMP
 */
class MIMP_Folder {

    /**
     * Cached results from the exists() function.
     *
     * @var array $existsResults
     */
    var $_existsResults = array();

    /**
     * Returns a reference to the global MIMP_Folder object, only
     * creating it if it doesn't already exist. This ensures that only
     * one MIMP_Folder instance is instantiated for any given session
     *
     * This method must be invoked as:
     *   $mimp_folder = &MIMP_Folder::singleton();
     *
     * @access public
     *
     * @return object MIMP_Folder  The MIMP_Folder instance.
     */
    function &singleton()
    {
        static $folder;

        if (!isset($folder)) {
            $folder = new MIMP_Folder();
        }

        return $folder;
    }

    /**
     * Create a new IMAP folder if it does not already exist, and
     * subcribe to it as well if requested.
     *
     * @access public
     *
     * @param resource $stream    A valid c-client stream.
     * @param string $folder      The full utf encoded folder to be created.
     * @param boolean $subscribe  A boolean describing whether or not to use
     *                            folder subscriptions.
     *
     * @return boolean  Whether or not the folder was successfully created.
     */
    function create($stream, $folder, $subscribe)
    {
        global $conf, $notification;

        /* Make sure we are not trying to create a duplicate folder */
        if ($this->exists($stream, $folder)) {
            $notification->push(sprintf(_("The folder \"%s\" already exists"), MIMP::displayFolder($folder)), 'horde.warning');
            return false;
        }

        /* Attempt to create the mailbox */
        if (!imap_createmailbox($stream, MIMP::serverString() . $folder)) {
            $notification->push(sprintf(_("The folder \"%s\" was not created. This is what the server said"), MIMP::displayFolder($folder)) .
                            ': ' . imap_last_error(), 'horde.error');
            return false;
        }

        /* Reset the folder cache. */
        if ($conf['server']['cache_folders']) {
            unset($_SESSION['mimp']['cache']['folder_cache']);
        }

        /* If the user uses SUBSCRIBE, then add to the subscribe list */
        if ($subscribe &&
            !imap_subscribe($stream, MIMP::serverString() . $folder)) {
            $notification->push(sprintf(_("The folder \"%s\" was created but you were not subscribed to it."), MIMP::displayFolder($folder)), 'horde.warning');
        } else {
            /* The folder creation has been successful. */
            $notification->push(sprintf(_("The folder \"%s\" was successfully created."), MIMP::displayFolder($folder)), 'horde.success');
        }

        /* Update the MIMP_Tree object. */
        require_once MIMP_BASE . '/lib/IMAP/Tree.php';
        $mimptree = &MIMP_Tree::singleton();
        if ($mimptree) {
            $mimptree->insert($folder);
        }

        return true;
    }

    /**
     * Find out if a specific folder exists or not.
     *
     * @param resource $stream           A valid c-client stream.
     * @param string $folder             The full utf encoded folder name to
     *                                   be checked.
     * @param optional boolean $recheck  Should we recheck the folder?
     *
     * @return boolean  Whether or not the folder exists.
     */
    function exists($stream, $folder, $recheck = false)
    {
        if ($recheck || !isset($this->_existsResults[$folder])) {
            $path = MIMP::serverString() . $folder;
            $res = imap_list($stream, '', $path);
            $this->_existsResults[$folder] = ($res ? in_array($path, $res) : false);
        }

        return $this->_existsResults[$folder];
    }

}
