<?php
/**
 * Functions required to start an MIMP session.
 *
 * $Horde: mimp/lib/Session.php,v 1.18 2005/01/03 14:35:16 jan Exp $
 *
 * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 1999-2005 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.18 $
 * @since   MIMP 0.0.1
 * @package MIMP
 */
class MIMP_Session {

    /**
     * Take information posted from a login attempt and try setting up
     * an initial MIMP session. Handle Horde authentication, if
     * required, and only do enough work to see if the user can log
     * in. This function should only be called once, when the user
     * first logs in.
     *
     * @access public
     *
     * @return mixed  True on success; string with the failure reason on
     *                failure.
     */
    function createSession($imapuser, $pass, $args)
    {
        global $conf;

        if (empty($imapuser) || empty($pass)) {
            return 'failed';
        }

        $server = isset($args['server']) ? $args['server'] : '';
        $realm = isset($args['realm']) ? $args['realm'] : '';
        $port = isset($args['port']) ? $args['port'] : '';
        $protocol = isset($args['protocol']) ? $args['protocol'] : '';
        $maildomain = isset($args['maildomain']) ? $args['maildomain'] : '';
        $namespace = isset($args['namespace']) ? $args['namespace'] : '';
        $folders = isset($args['folders']) ? $args['folders'] : '';

        $_SESSION['mimp'] = array();
        $_SESSION['mimp']['user'] = $imapuser;

        /* Set the login flag. */
        MIMP_Session::loginFlag(true);

        /* Run the username through virtualhost expansion functions if
         * necessary. */
        if (!empty($conf['hooks']['vinfo'])) {
            require_once HORDE_BASE . '/config/hooks.php';
            if (function_exists('_mimp_hook_vinfo')) {
                $_SESSION['mimp']['user'] = call_user_func('_mimp_hook_vinfo');
            }
        }

        $_SESSION['mimp']['pass'] = Secret::write(Secret::getKey('mimp'), $pass);

        /* We might need to override some of the defaults with
         * environmental settings. */
        global $servers;
        require MIMP_BASE . '/config/servers.php';

        if (Auth::isAuthenticated()) {
            $_SESSION['mimp']['uniquser'] = Auth::getAuth();
        } elseif (($conf['server']['server_list'] != 'none') &&
                  !empty($servers[$server]['realm'])) {
            $_SESSION['mimp']['uniquser'] = $_SESSION['mimp']['user'] . '@' . $servers[$server]['realm'];
        } elseif (!empty($realm)) {
            $_SESSION['mimp']['uniquser'] = $_SESSION['mimp']['user'] . '@' . $realm;
        } else {
            $_SESSION['mimp']['uniquser'] = $_SESSION['mimp']['user'];
        }

        if (($conf['server']['server_list'] != 'none') &&
            isset($servers[$server]) &&
            is_array($servers[$server])) {
            $settings = $servers[$server];
            $_SESSION['mimp']['server'] = $settings['server'];
            $_SESSION['mimp']['port'] = $settings['port'];
            $_SESSION['mimp']['protocol'] = $settings['protocol'];
            $_SESSION['mimp']['maildomain'] = $settings['maildomain'];
            $_SESSION['mimp']['namespace'] = $settings['namespace'];
            $_SESSION['mimp']['folders'] = $settings['folders'];

            if (($conf['mailer']['type'] == 'smtp') &&
                !empty($settings['smtphost'])) {
                $_SESSION['mimp']['smtphost'] = $settings['smtphost'];
            }
        } else {
            foreach ($servers as $key => $curServer) {
                if (!isset($server_key) && substr($key, 0, 1) != '_') {
                    $server_key = $key;
                }
                if (MIMP::isPreferredServer($curServer, $key)) {
                    $server_key = $key;
                    break;
                }
            }
            if ($conf['server']['change_server']) {
                $_SESSION['mimp']['server'] = $server;
            } else {
                $_SESSION['mimp']['server'] = $servers[$server_key]['server'];
            }
            if ($conf['server']['change_port']) {
                $_SESSION['mimp']['port'] = $port;
            } else {
                $_SESSION['mimp']['port'] = $servers[$server_key]['port'];
            }
            if ($conf['server']['change_protocol']) {
                $_SESSION['mimp']['protocol'] = $protocol;
            } else {
                $_SESSION['mimp']['protocol'] = $servers[$server_key]['protocol'];
            }
            if ($conf['server']['change_folders']) {
                $_SESSION['mimp']['folders'] = $folders;
            } else {
                $_SESSION['mimp']['folders'] = $servers[$server_key]['folders'];
            }
            $_SESSION['mimp']['maildomain'] = $maildomain;
            $_SESSION['mimp']['namespace'] = $namespace;
        }

        $protocols = explode('/', $_SESSION['mimp']['protocol']);
        if (is_array($protocols)) {
            $_SESSION['mimp']['base_protocol'] = $protocols[0];
        } else {
            $_SESSION['mimp']['base_protocol'] = $_SESSION['mimp']['protocol'];
        }

        $_SESSION['mimp']['mailbox'] = '';

        /* Try to authenticate with the given information. */
        $auth_mimp = &Auth::singleton(array('mimp', 'mimp'));
        $auth_mimp->authenticateOptions(array('flags' => OP_HALFOPEN));
        if ($auth_mimp->authenticate(null, null, true) === true) {
            global $prefs;

            /* Set the session variables. These are cached. */
            $_SESSION['mimp']['mailbox'] = $prefs->getValue('mailbox');

            /* IMAP specific variables. */
            if ($_SESSION['mimp']['base_protocol'] != 'pop3') {
                /* Get the delimiter value (per RFC 3501 [6.3.8]). */
                $box = @imap_getmailboxes($_SESSION['mimp']['stream'], MIMP::serverString(), '');
                if (empty($box)) {
                    $box = @imap_getmailboxes($_SESSION['imp']['stream'], IMP::serverString(), 'INBOX');
                }
                $_SESSION['mimp']['delimiter'] = !empty($box[0]->delimiter) ? $box[0]->delimiter : '/';
            }

            return true;
        } else {
            return 'failed';
        }
    }

    /**
     * Perform MIMP login tasks.
     *
     * @access public
     */
    function loginTasks()
    {
        if (MIMP_Session::loginFlag()) {
            if (empty($_SESSION['mimp']['stream'])) {
                MIMP::checkAuthentication(OP_HALFOPEN, true);
            }

            MIMP_Session::loginFlag(false);
        }
    }

    /**
     * Either sets or checks the value of the login flag.
     *
     * @access public
     *
     * @param optional boolean $set  The value of the flag.
     *
     * @return boolean  The value of the flag.
     */
    function loginFlag($set = null)
    {
        if (!is_null($set)) {
            if ($set) {
                $_SESSION['mimp']['_login'] = true;
            } else {
                unset($_SESSION['mimp']['_login']);
            }
        }

        return isset($_SESSION['mimp']['_login']);
    }

}
